/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */

package org.apache.shindig.gadgets.js;

import com.google.common.annotations.VisibleForTesting;
import com.google.common.collect.Lists;
import com.google.inject.Inject;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shindig.common.uri.Uri;
import org.apache.shindig.gadgets.uri.JsUriManager;
import org.apache.shindig.gadgets.uri.JsUriManager.JsUri;

import java.util.Collection;
import java.util.List;

/**
 * Injects a global ___jsl variable with information about the JS request.
 *
 * Used when loading embedded JS configuration in core.config/config.js.
 */
public class AddJslInfoVariableProcessor implements JsProcessor {
  private static final String CODE_ID = "[jsload-code-info]";

  @VisibleForTesting
  static final String BASE_HINT_TEMPLATE = "window['___jsl'] = window['___jsl'] || {};";

  @VisibleForTesting
  static final String FEATURES_HINT_TEMPLATE = "window['___jsl']['f'] = [%s];";

  @VisibleForTesting
  static final String URL_HINT_TEMPLATE = "window['___jsl']['u'] = '%s';";

  private final JsUriManager jsUriManager;

  @Inject
  public AddJslInfoVariableProcessor(JsUriManager jsUriManager) {
    this.jsUriManager = jsUriManager;
  }

  public boolean process(JsRequest jsRequest, JsResponseBuilder builder) {
    JsUri jsUri = jsRequest.getJsUri();
    if (!jsUri.isNohint()) {
      String uri = StringEscapeUtils.escapeEcmaScript(getUri(jsUri));
      String features = getFeatures(jsUri);
      builder.prependJs(String.format(FEATURES_HINT_TEMPLATE, features), CODE_ID);
      if (uri != null && !"".equals(uri)) {
        builder.prependJs(String.format(URL_HINT_TEMPLATE, uri), CODE_ID);
      }
      builder.prependJs(BASE_HINT_TEMPLATE, CODE_ID);
    }
    return true;
  }

  protected String getUri(JsUri jsUri) {
    Uri uri = jsUri.getOrigUri();
    if (uri == null || jsUri.isJsload()) {
      JsUri auxUri = new JsUri(jsUri);
      auxUri.setJsload(false);
      if (jsUri.isJsload()) {
        auxUri.setNohint(true);
      }
      uri = jsUriManager.makeExternJsUri(auxUri);
    }
    return uri.toString();
  }

  private String getFeatures(JsUri jsUri) {
    Collection<String> libs = jsUri.getLibs();
    List<Object> features = Lists.newArrayList();
    for (String lib : libs) {
      features.add('\'' + StringEscapeUtils.escapeEcmaScript(lib) + '\'');
    }
    return StringUtils.join(features, ",");
  }
}
